# This file is a part of the AnyBlok project
#
#    Copyright (C) 2014 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from sqlalchemy import Table, Column, ForeignKey
from anyblok import Declarations


FieldException = Declarations.Exception.FieldException


@Declarations.register(Declarations.RelationShip)
class Many2Many(Declarations.RelationShip):
    """ Define a relationship attribute on the model

    ::

        @register(Model)
        class TheModel:

            relationship = Many2Many(label="The relationship",
                                     model=Model.RemoteModel,
                                     join_table="many2many table",
                                     remote_column="The remote column",
                                     m2m_remote_column="Name in many2many"
                                     local_column="local primary key"
                                     m2m_local_column="Name in many2many"
                                     many2many="themodels")

    if the join_table is not defined, then the table join is
        "join_'local table'_and_'remote table'"

    If the remote_column are not define then, the system take the primary key
    of the remote model

    if the local_column are not define the take the primary key of the local
        model

    :param model: the remote model
    :param join_table: the many2many table to join local and remote models
    :param remote_column: the column name on the remote model
    :param m2m_remote_column: the column name to remote model in m2m table
    :param local_column: the column on the model
    :param m2m_local_column: the column name to local model in m2m table
    :param many2many: create the opposite many2many on the remote model
    """

    def __init__(self, **kwargs):
        super(Many2Many, self).__init__(**kwargs)

        self.join_table = None
        if 'join_table' in kwargs:
            self.join_table = self.kwargs.pop('join_table')

        self.remote_column = None
        if 'remote_column' in kwargs:
            self.remote_column = self.kwargs.pop('remote_column')
            self.kwargs['info']['remote_column'] = self.remote_column

        self.m2m_remote_column = None
        if 'm2m_remote_column' in kwargs:
            self.m2m_remote_column = self.kwargs.pop('m2m_remote_column')

        self.local_column = None
        if 'local_column' in kwargs:
            self.local_column = self.kwargs.pop('local_column')
            self.kwargs['info']['local_column'] = self.local_column

        self.m2m_local_column = None
        if 'm2m_local_column' in kwargs:
            self.m2m_local_column = self.kwargs.pop('m2m_local_column')

        if 'many2many' in kwargs:
            self.kwargs['backref'] = self.kwargs.pop('many2many')
            self.kwargs['info']['remote_name'] = self.kwargs['backref']

    def get_m2m_column_info(self, tablename, properties, column, m2m_column):
        if column is None:
            column = self.find_primary_key(properties)
        elif column not in properties:
            raise FieldException("%r does not exist in %r" % (column,
                                                              tablename))

        if m2m_column is None:
            m2m_column = '%s_%s' % (tablename, column)

        foreignkey = '%s.%s' % (tablename, column)

        return m2m_column, properties[column].native_type(), foreignkey

    def get_sqlalchemy_mapping(self, registry, namespace, fieldname,
                               properties):
        """ Create the relationship

        :param registry: the registry which load the relationship
        :param namespace: the name space of the model
        :param fieldname: fieldname of the relationship
        :param properties: the properties known
        :rtype: Many2One relationship
        """
        self.check_existing_remote_model(registry)
        remote_properties = registry.loaded_namespaces_first_step.get(
            self.get_registry_name())
        local_properties = registry.loaded_namespaces_first_step.get(namespace)

        local_tablename = properties['__tablename__']
        remote_tablename = self.get_tablename(registry)
        if self.join_table is None:
            self.join_table = 'join_%s_and_%s' % (local_tablename,
                                                  remote_tablename)

        if self.join_table not in registry.declarativebase.metadata.tables:
            rname, rtype, rfk = self.get_m2m_column_info(
                remote_tablename, remote_properties, self.remote_column,
                self.m2m_remote_column)

            lname, ltype, lfk = self.get_m2m_column_info(
                local_tablename, local_properties, self.local_column,
                self.m2m_local_column)

            Table(self.join_table, registry.declarativebase.metadata,
                  Column(lname, ltype, ForeignKey(lfk)),
                  Column(rname, rtype, ForeignKey(rfk)))

        self.kwargs['secondary'] = self.join_table

        return super(Many2Many, self).get_sqlalchemy_mapping(
            registry, namespace, fieldname, properties)



# This file is a part of the AnyBlok project
#
#    Copyright (C) 2014 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from anyblok.tests.testcase import TestCase
from anyblok import Declarations
Field = Declarations.Field
Column = Declarations.Column
RelationShip = Declarations.RelationShip
register = Declarations.register
unregister = Declarations.unregister
FieldException = Declarations.Exception.FieldException


class OneField(Field):
    pass


class TestField(TestCase):

    def test_forbid_instance(self):
        try:
            Field()
            self.fail("Field mustn't be instanciated")
        except FieldException:
            pass

    def test_without_label(self):
        register(Field, cls_=OneField, name_='RealField')
        field = Field.RealField()
        field.get_sqlalchemy_mapping(None, None, 'a_field', None)
        self.assertEqual(field.label, 'A field')

    def test_add_interface(self):
        register(Field, cls_=OneField, name_='OneField')
        self.assertEqual('Field', Field.OneField.__declaration_type__)
        dir(Declarations.Field.OneField)

    def test_add_interface_with_decorator(self):

        @register(Field)
        class OneDecoratorField(OneField):
            pass

        self.assertEqual('Field', Field.OneDecoratorField.__declaration_type__)
        dir(Declarations.Field.OneDecoratorField)

    def test_add_same_interface(self):

        register(Field, cls_=OneField, name_="SameField")

        try:
            @register(Field)
            class SameField(OneField):
                pass

            self.fail('No watch dog to add 2 same field')
        except FieldException:
            pass

    def test_remove_interface(self):

        register(Field, cls_=OneField, name_="Field2Remove")
        try:
            unregister(Field.Field2Remove, OneField)
            self.fail('No watch dog to remove field')
        except FieldException:
            pass



# This file is a part of the AnyBlok project
#
#    Copyright (C) 2014 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from anyblok import Declarations
from .declarations import DeclarationsException
from anyblok.environment import EnvironmentManager, EnvironmentException


@Declarations.add_declaration_type()
class Exception:
    """ Adapter to Exception Class

    The Exception class is used to define the type of Declarations Exception

    Add new Exception type::

        @Declarations.register(Declarations.Exception)
        class MyException:
            pass

    Removing the exception is forbidden because it can be used
    """

    @classmethod
    def register(self, parent, name, cls_, **kwargs):
        """ add new sub registry in the registry

        :param parent: Existing declaration
        :param name: Name of the new declaration to add it
        :param cls_: Class to add in the declaration
        :exception: DeclarationsException
        """
        _registryname = parent.__registry_name__ + '.' + name
        if hasattr(parent, name) and not EnvironmentManager.get('reload',
                                                                False):
            raise DeclarationsException(
                "The Exception %r already exist" % _registryname)

        setattr(cls_, '__registry_name__', _registryname)
        setattr(cls_, '__declaration_type__', parent.__declaration_type__)
        setattr(parent, name, cls_)

    @classmethod
    def unregister(self, entry, cls_):
        """ Forbidden method

        :exception: DeclarationsException
        """
        raise DeclarationsException("Removing an exception is forbidden")


Declarations.register(Declarations.Exception, cls_=DeclarationsException)
Declarations.register(Declarations.Exception, cls_=EnvironmentException)



import anyblok
from anyblok._logging import log
from anyblok._argsparse import ArgsParseManager
from sqlalchemy import create_engine
from contextlib import contextmanager


class SqlAlchemyPostgres:
    """ Postgres adapter for database management"""

    __interface__ = 'postgres'

    @contextmanager
    def cnx(self):
        """ Context manager to get a connection to database """
        cnx = None
        try:
            postgres = ArgsParseManager.get_url(dbname='postgres')
            engine = create_engine(postgres)
            cnx = engine.connect()
            cnx.execute("commit")
            yield cnx
            cnx.execute("commit")
        except Exception:
            if cnx:
                cnx.execute("rollback")
            raise
        finally:
            if cnx:
                cnx.close()

    @log(withargs=True)
    def createdb(self, dbname):
        """ Create a database

        :param dbname: database name to create
        """
        with self.cnx() as conn:
            conn.execute("""create database "%s";""" % dbname)

    @log(withargs=True)
    def dropdb(self, dbname):
        """ Drop a database

        :param dbname: database name to drop
        """
        with self.cnx() as conn:
            conn.execute("""drop database "%s";""" % dbname)

    def listdb(self):
        """ list database

        :rtype: list of database name
        """
        select_clause = "select pg_db.datname"
        from_clause = " from pg_database pg_db"
        where_clause = " where pg_db.datname not in (%s)"
        values = ', '.join(["'template0'", "'template1'", "'postgres'"])

        username = ArgsParseManager.get('dbusername')
        if username:
            from_clause += ", pg_user pg_u"
            where_clause += " and pg_u.username = %s"
            values = [values, username]
            where_clause += " and pg_db.datdba=pg_u.usesysid"

        query = (select_clause + from_clause + where_clause) % values
        with self.cnx() as conn:
            res = conn.execute(query)

        return [x[0] for x in res.fetchall()]


anyblok.BDD['postgres'] = SqlAlchemyPostgres()



# -*- coding: utf-8 -*-
# This file is a part of the AnyBlok project
#
#    Copyright (C) 2014 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
import anyblok
from sys import modules
from os.path import splitext, split
from os import listdir
from importlib import import_module
from .common import python_version


@anyblok.Declarations.register(anyblok.Declarations.Exception)
class ImportManagerException(Exception):
    """ Exception for Import Manager """


class ImportManager:
    """ Use to import the blok or reload the blok imports

        Add a blok and imports its modules::

            blok = ImportManager.add('my blok')
            blok.imports()

        Reload the modules of a blok::

            if ImportManager.has('my blok'):
                blok = ImportManager.get('my blok')
                blok.reload()
                # import the unimported module
    """
    modules = {}

    @classmethod
    def add(cls, blok):
        """ Store the blok so that we know which bloks to reload if needed

        :param blok: name of the blok to add
        :rtype: loader instance
        :exception: ImportManagerException
        """
        from anyblok.blok import BlokManager
        if cls.has(blok):
            return cls.get(blok)

        if not BlokManager.has(blok):
            raise ImportManagerException("Unexisting blok")

        loader = Loader(blok)
        cls.modules[blok] = loader
        return loader

    @classmethod
    def get(cls, blok):
        """ Return the module imported for this blok

        :param blok: name of the blok to add
        :rtype: loader instance
        :exception: ImportManagerException
        """
        if not cls.has(blok):
            raise ImportManagerException('Unexisting blok %r' % blok)
        return cls.modules[blok]

    @classmethod
    def has(cls, blok):
        """ Return True if the blok was imported

        :param blok: name of the blok to add
        :rtype: boolean
        """
        return blok in cls.modules


class Loader:

    def __init__(self, blok):
        self.blok = blok
        self.import_known = []

    def imports(self):
        """ Imports modules and / or packages listed in the blok path"""
        from anyblok.blok import BlokManager
        from anyblok.registry import RegistryManager

        RegistryManager.init_blok(self.blok)
        b = BlokManager.get(self.blok)
        main_path = modules[b.__module__].__file__
        path, init = split(main_path)

        mods = [x for x in listdir(path) if x != init and x[0] != '.']
        for module in mods:
            module_name = b.__module__ + '.' + splitext(module)[0]
            try:
                import_module(module_name)
            except ImportError:
                pass

        self.import_known = [x for x in modules.keys()
                             if b.__module__ + '.' in x]

    def reload(self):
        """ Reload all the imports for this module

        :exception: ImportManagerException
        """
        isimp = False
        try:
            from importlib import reload as reload_module
        except ImportError:
            isimp = True
            from imp import reload as reload_module

        from anyblok.blok import BlokManager
        from anyblok.registry import RegistryManager
        from anyblok.environment import EnvironmentManager

        b = BlokManager.get(self.blok)
        b.clean_before_reload()
        RegistryManager.init_blok(self.blok)
        main_path = modules[b.__module__].__file__
        path, init = split(main_path)
        mods = [x for x in listdir(path) if x != init and x[0] != '.']

        try:
            EnvironmentManager.set('reload', True)

            for module in mods:
                module_name = b.__module__ + '.' + splitext(module)[0]
                if module_name in self.import_known:
                    continue

                try:
                    import_module(module_name)
                except ImportError:
                    pass

            self.import_known.sort()
            for module in self.import_known:
                try:
                    if isimp:
                        module2load = modules[module]
                    elif python_version() == (3, 3):
                        module2load = module
                    elif python_version() >= (3, 4):
                        module2load = modules[module]
                    else:
                        raise ImportManagerException(
                            "Unknow action to do to reload module %r" % module)

                    reload_module(module2load)
                except ImportError:
                    pass
        finally:
            EnvironmentManager.set('reload', False)

        self.import_known = [x for x in modules.keys()
                             if b.__module__ + '.' in x]



# This file is a part of the AnyBlok project
#
#    Copyright (C) 2014 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
import anyblok
from os.path import join
from anyblok import _argsparse
from anyblok._argsparse import (ArgsParseManager,
                                add_configuration_file,
                                add_database,
                                add_install_bloks,
                                add_uninstall_bloks,
                                add_update_bloks,
                                add_interpreter,
                                add_logging,
                                add_schema)
from anyblok.tests.testcase import TestCase
from anyblok import Declarations


ArgsParseManagerException = Declarations.Exception.ArgsParseManagerException
old_getParser = _argsparse.getParser


def fnct_argsparse(parser, default):
    default.update({'test': None})


def fnct_other_argsparse(parser, default):
    default.update({'test': None})


class MockArgParseArguments:

    def __init__(self, configfile=None, args=None, kwargs=None):
        if configfile:
            cfile = join(anyblok.__path__[0], 'tests', configfile)
            self.configfile = cfile
        else:
            self.configfile = None
        self.args = args
        self.kwargs = kwargs

    def _get_args(self):
        return self.args

    def _get_kwargs(self):
        if self.kwargs is None:
            return tuple()
        return self.kwargs.items()


class MockArguments:

    configfile = None
    vals = {'var1': 'val1',
            'var2': 'val2',
            'var3': 'val3'}

    def _get_kwargs(self):
        return self.vals.items()

    def _get_args(self):
        return False


class MockArgumentParser:

    def __init__(self, *args, **kwargs):
        self.args = MockArguments()

    def add_argument_group(self, *args, **kwargs):
        return MockArgumentParser()

    def parse_args(self, *args, **kwargs):
        return self.args

    def add_argument(self, *args, **kwargs):
        pass


class TestArgsParseManager(TestCase):

    @classmethod
    def setUpClass(cls):
        super(TestArgsParseManager, cls).setUpClass()

        def getParser(*args, **kwargs):
            return MockArgumentParser()

        _argsparse.getParser = getParser

    @classmethod
    def tearDownClass(cls):
        super(TestArgsParseManager, cls).tearDownClass()
        _argsparse.getParser = old_getParser

    def setUp(self):
        super(TestArgsParseManager, self).setUp()
        ArgsParseManager.groups = {}
        ArgsParseManager.labels = {}
        ArgsParseManager.configuration = {}

    def assertAdded(self, group, part='AnyBlok', label=None, function_=None):
        self.assertEqual(ArgsParseManager.groups[part][group], [function_])

        if label:
            self.assertEqual(ArgsParseManager.labels[part][group], label)

    def test_add(self):
        ArgsParseManager.add('new-group', function_=fnct_argsparse)
        self.assertAdded('new-group', function_=fnct_argsparse)

    def test_add_with_label(self):
        ArgsParseManager.add(
            'new-group', label="One label", function_=fnct_argsparse)
        self.assertAdded(
            'new-group', label="One label", function_=fnct_argsparse)

    def test_add_other_part(self):
        ArgsParseManager.add(
            'new-group', part='other', function_=fnct_argsparse)
        self.assertAdded('new-group', part='other', function_=fnct_argsparse)

    def test_add_other_part_with_label(self):
        ArgsParseManager.add('new-group', part='other', label="One label",
                             function_=fnct_argsparse)
        self.assertAdded('new-group', part='other', label="One label",
                         function_=fnct_argsparse)

    def test_add_decorator(self):

        @ArgsParseManager.add('new-group')
        def fnct(parser, default):
            default.update({'other test': True})

        self.assertAdded('new-group', function_=fnct)

    def test_get(self):
        option = 'My option'
        ArgsParseManager.configuration['option'] = option
        res = ArgsParseManager.get('option')
        self.assertEqual(option, res)

    def test_get_use_default_value(self):
        option = 'My option by default'
        res = ArgsParseManager.get('option', option)
        self.assertEqual(option, res)

    def test_get_url(self):
        ArgsParseManager.configuration.update(dict(
            dbname='anyblok',
            dbdrivername='postgres',
            dbhost='localhost',
            dbusername=None,
            dbpassword=None,
            dbport=None,
        ))
        ArgsParseManager.get_url()

    def test_get_url_without_drivername(self):
        ArgsParseManager.configuration.update(dict(
            dbname=None,
            dbdrivername=None,
            dbhost=None,
            dbusername=None,
            dbpassword=None,
            dbport=None,
        ))
        try:
            ArgsParseManager.get_url()
            self.fail("No watchdog found for no drivername")
        except ArgsParseManagerException:
            pass

    def test_merge_for_one_part(self):
        ArgsParseManager.add('new-group', function_=fnct_argsparse)
        ArgsParseManager.add('new-group', function_=fnct_other_argsparse)
        ArgsParseManager.add('old-group', function_=fnct_argsparse)
        ArgsParseManager.add('old-group', part='other',
                             function_=fnct_argsparse)
        groups = ArgsParseManager._merge_groups('AnyBlok')
        self.assertEqual(groups, {
            'new-group': [fnct_argsparse, fnct_other_argsparse],
            'old-group': [fnct_argsparse]})

    def test_merge_for_more_parts(self):
        ArgsParseManager.add('new-group', function_=fnct_argsparse)
        ArgsParseManager.add('new-group', function_=fnct_other_argsparse)
        ArgsParseManager.add('old-group', function_=fnct_argsparse)
        ArgsParseManager.add('old-group', part='other',
                             function_=fnct_other_argsparse)
        groups = ArgsParseManager._merge_groups('AnyBlok', 'other')
        self.assertEqual(groups, {
            'new-group': [fnct_argsparse, fnct_other_argsparse],
            'old-group': [fnct_argsparse, fnct_other_argsparse]})

    def test_merge_no_parts(self):
        try:
            ArgsParseManager._merge_groups()
            self.fail('No watchdog to merge no part')
        except ArgsParseManagerException:
            pass

    def test_merge_inexisting_part(self):
        ArgsParseManager._merge_groups('other')

    def test_merge_label(self):
        ArgsParseManager.add('new-group', label="Label 1",
                             function_=fnct_other_argsparse)
        ArgsParseManager.add('old-group', label="Label 2", part='other',
                             function_=fnct_argsparse)
        labels = ArgsParseManager._merge_labels('AnyBlok')
        self.assertEqual(labels, {'new-group': "Label 1"})

    def test_merge_label_with_more_parts(self):
        ArgsParseManager.add('new-group', label="Label 1",
                             function_=fnct_other_argsparse)
        ArgsParseManager.add('old-group', label="Label 2", part='other',
                             function_=fnct_argsparse)
        labels = ArgsParseManager._merge_labels('AnyBlok', 'other')
        self.assertEqual(labels, {'new-group': "Label 1",
                                  'old-group': "Label 2"})

    def test_merge_labels_with_no_parts(self):
        try:
            ArgsParseManager._merge_labels()
            self.fail('No watchdog to merge no part')
        except ArgsParseManagerException:
            pass

    def test_merge_labels_inexisting_part(self):
        ArgsParseManager._merge_labels('other')

    def test_remove(self):
        ArgsParseManager.add('new-group', function_=fnct_argsparse)
        ArgsParseManager.remove('new-group', function_=fnct_argsparse)
        self.assertEqual(ArgsParseManager.groups['AnyBlok']['new-group'], [])

    def test_remove_other_part(self):
        ArgsParseManager.add('new-group', part='other',
                             function_=fnct_argsparse)
        ArgsParseManager.remove('new-group', part='other',
                                function_=fnct_argsparse)
        self.assertEqual(ArgsParseManager.groups['other']['new-group'], [])

    def test_remove_more_function(self):
        ArgsParseManager.add('new-group', function_=fnct_argsparse)
        ArgsParseManager.add('new-group', function_=fnct_other_argsparse)
        ArgsParseManager.remove('new-group', function_=fnct_argsparse)
        self.assertEqual(ArgsParseManager.groups['AnyBlok']['new-group'],
                         [fnct_other_argsparse])

    def test_remove_label(self):
        ArgsParseManager.add(
            'new-group', label="One label", function_=fnct_argsparse)
        ArgsParseManager.remove_label('new-group')
        try:
            ArgsParseManager.labels['AnyBlok']['new-group']
            self.fail("Label doesn't remove")
        except:
            pass

    def test_remove_label_other_part(self):
        ArgsParseManager.add('new-group', part='other', label="One label",
                             function_=fnct_argsparse)
        ArgsParseManager.remove_label('new-group', part='other')
        try:
            ArgsParseManager.labels['other']['new-group']
            self.fail("Label doesn't remove")
        except:
            pass

    def test_logging(self):
        ArgsParseManager.configuration.update(dict(
            logging_level='info',
            logging_mode='console',
            logging_filename=None,
            logging_socket=None,
            logging_facility=None,
        ))
        ArgsParseManager.init_logger()

    def test_logging_with_kwargs(self):
        ArgsParseManager.configuration.update(dict(
            logging_level='info',
            logging_mode='file',
            logging_filename=None,
            logging_socket=None,
            logging_facility=None,
        ))
        ArgsParseManager.init_logger(mode='console')

    def test_load_without_argsparse_groupes(self):
        self.assertEqual(ArgsParseManager.load(), None)

    def test_empty_parse_option(self):
        args = MockArgParseArguments()
        ArgsParseManager.parse_options(args, ['AnyBlok'])
        self.assertEqual(ArgsParseManager.configuration, {})

    def test_parse_option(self):
        kwargs = {'test': 'value'}
        args = MockArgParseArguments(configfile="mock_configuration_file.cfg",
                                     kwargs=kwargs)
        ArgsParseManager.parse_options(args, ['AnyBlok'])
        kwargs.update({
            'dbname': 'anyblok',
            'dbdrivername': 'postgres',
            'dbusername': '',
            'dbpassword': '',
            'dbhost': 'localhost',
            'dbport': '',
            'wsgi_port': '8080',
        })
        self.assertEqual(ArgsParseManager.configuration, kwargs)

    def test_parse_option_configuration(self):
        args = MockArgParseArguments(configfile="mock_configuration_file.cfg")
        ArgsParseManager.parse_options(args, ['AnyBlok'])
        self.assertEqual(ArgsParseManager.configuration, {
            'dbname': 'anyblok',
            'dbdrivername': 'postgres',
            'dbusername': '',
            'dbpassword': '',
            'dbhost': 'localhost',
            'dbport': '',
            'wsgi_port': '8080',
        })

    def test_parse_option_kwargs(self):
        kwargs = {'test': 'value'}
        args = MockArgParseArguments(kwargs=kwargs)
        ArgsParseManager.parse_options(args, ['AnyBlok'])
        self.assertEqual(ArgsParseManager.configuration, kwargs)

    def test_parse_option_args(self):
        args = ('test',)
        args = MockArgParseArguments(args=args)
        try:
            ArgsParseManager.parse_options(args, ['AnyBlok'])
            self.fail("No watchdog for positionnal arguments")
        except ArgsParseManagerException:
            pass

    def test_load_with_argsparse_groupes(self):
        ArgsParseManager.load(argsparse_groups=['install-bloks'])
        self.assertEqual(ArgsParseManager.configuration, MockArguments.vals)

    def test_load_with_bad_argsparse_groupes(self):
        ArgsParseManager.load(argsparse_groups=['bad-groups'])
        self.assertEqual(ArgsParseManager.configuration, MockArguments.vals)


class TestArgsParseOption(TestCase):

    @classmethod
    def setUpClass(cls):
        super(TestArgsParseOption, cls).setUpClass()
        cls.parser = MockArgumentParser()
        cls.group = cls.parser.add_argument_group('label')
        cls.configuration = {}
        cls.function = {
            'add_configuration_file': add_configuration_file,
            'add_database': add_database,
            'add_install_bloks': add_install_bloks,
            'add_uninstall_bloks': add_uninstall_bloks,
            'add_update_bloks': add_update_bloks,
            'add_interpreter': add_interpreter,
            'add_logging': add_logging,
            'add_schema': add_schema,
        }

    def test_add_configuration_file(self):
        self.function['add_configuration_file'](self.parser,
                                                self.configuration)

    def test_add_database(self):
        self.function['add_database'](self.group, self.configuration)

    def test_add_install_bloks(self):
        self.function['add_install_bloks'](self.parser, self.configuration)

    def test_add_uninstall_bloks(self):
        self.function['add_uninstall_bloks'](self.parser, self.configuration)

    def test_add_update_bloks(self):
        self.function['add_update_bloks'](self.parser, self.configuration)

    def test_add_interpreter(self):
        self.function['add_interpreter'](self.parser, self.configuration)

    def test_add_logging(self):
        self.function['add_logging'](self.group, self.configuration)

    def test_add_schema(self):
        self.function['add_schema'](self.group, self.configuration)



# flake8: noqa
# This file is a part of the AnyBlok project
#
#    Copyright (C) 2014 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from . import integer
from . import small_integer
from . import big_integer
from . import _float
from . import _decimal
from . import boolean
from . import string
from . import ustring
from . import text
from . import utext
from . import date
from . import _datetime
from . import interval
from . import _time
from . import large_binary
from . import selection
from . import _json



# -*- coding: utf-8 -*-
# This file is a part of the AnyBlok project
#
#    Copyright (C) 2014 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from . import release


PROMPT = "%(processName)s - %(version)s"


def start(processName, version=release.version, prompt=PROMPT,
          argsparse_groups=None, parts_to_load=None, logger=None):
    """ Function which initialize the application

    ::

        registry = start('My application',
                         argsparse_groups=['config', 'database'],
                         parts_to_load=['AnyBlok'])

    :param processName: Name of the application
    :param version: Version of the application
    :param prompt: Prompt message for the help
    :param argsparse_groups: list of the group of option for argparse
    :param parts_to_load: group of blok to load
    :param logger: option to configure  logging
    :rtype: registry if the database name is in the configuration
    """
    from .blok import BlokManager
    from ._argsparse import ArgsParseManager
    from .registry import RegistryManager

    if parts_to_load is None:
        parts_to_load = ['AnyBlok']

    BlokManager.load(*parts_to_load)
    description = prompt % {'processName': processName, 'version': version}
    if argsparse_groups is not None:
        ArgsParseManager.load(description=description,
                              argsparse_groups=argsparse_groups,
                              parts_to_load=parts_to_load)

    if logger is None:
        logger = {}
    ArgsParseManager.init_logger(**logger)
    dbname = ArgsParseManager.get('dbname')
    if not dbname:
        return None

    registry = RegistryManager.get(dbname)
    registry.commit()
    return registry


BDD = {
}


from .declarations import Declarations  # noqa
from . import exception  # noqa
from . import databases  # noqa
from . import core  # noqa
from . import field  # noqa
from . import column  # noqa
from . import relationship  # noqa
from . import model  # noqa
from . import mixin  # noqa
